@@ -6,6 +6,7 @@
 #include <libgnomeprint/gnome-print-i18n.h>
 #include <libgnomeprint/gnome-font.h>
 #include <libgnomeprint/gnome-font-private.h>
+#include <libgnomeprint/gp-unicode.h>
 #include <libgnomeprint/gp-ps-unicode.h>
 #include "parseAFM.h"
 
@@ -74,6 +75,13 @@ gnome_font_face_init (GnomeFontFace * face)
 	face->private = g_new0 (GnomeFontFacePrivate, 1);
 }
 
+static gboolean
+gff_free_privencoding (gpointer key, gpointer value, gpointer data)
+{
+	g_free (key);
+	return TRUE;
+}
+
 static void
 gnome_font_face_destroy (GtkObject * object)
 {
@@ -95,6 +103,10 @@ gnome_font_face_destroy (GtkObject * object)
 		if (priv->alias) g_free (priv->alias);
 		if (priv->glyphs) g_free (priv->glyphs);
 		if (priv->glyphmap) g_hash_table_destroy (priv->glyphmap);
+		if (priv->privencoding) {
+			g_hash_table_foreach_remove (priv->privencoding, gff_free_privencoding, NULL);
+			g_hash_table_destroy (priv->privencoding);
+		}
 		if (priv->kerns) g_free (priv->kerns);
 		if (priv->ligs) {
 			gint i;
@@ -407,13 +419,43 @@ gnome_font_face_get_glyph_width (const GnomeFontFace * face, gint glyph)
 const gchar *
 gnome_font_face_get_sample (const GnomeFontFace * face)
 {
-	if (strcmp (face->private->familyname, "Symbol") == 0) {
-		static guchar s[] = {0xce, 0x93, 0xce, 0xb5,
-				     0xce, 0xb9, 0xce, 0xac,
-				     0x20, 0xcf, 0x83, 0xce,
-				     0xb1, 0xcf, 0x82, 0x00};
-		return s;
+	GnomeFontFacePrivate * priv;
+	static gchar s[256];
+	gchar *p;
+	gint i;
+
+	g_return_val_if_fail (face != NULL, NULL);
+	g_return_val_if_fail (GNOME_IS_FONT_FACE (face), NULL);
+	g_return_val_if_fail (GFF_METRICS (face), NULL);
+
+	/* This is slow and experimental */
+
+	priv = face->private;
+
+	if (!g_hash_table_lookup (priv->glyphmap, GINT_TO_POINTER ('a')) ||
+	    !g_hash_table_lookup (priv->glyphmap, GINT_TO_POINTER ('e')) ||
+	    !g_hash_table_lookup (priv->glyphmap, GINT_TO_POINTER ('q'))) {
+		/* We do not have a, e and q - test for greek letters */
+		if (g_hash_table_lookup (priv->glyphmap, GINT_TO_POINTER (0x0391)) &&
+		    g_hash_table_lookup (priv->glyphmap, GINT_TO_POINTER (0x0395)) &&
+		    g_hash_table_lookup (priv->glyphmap, GINT_TO_POINTER (0x03a8))) {
+			p = s;
+			for (i = 0; i < 16; i++ ) {
+				p += g_unichar_to_utf8 (0x0391 + i, p);
+				p += g_unichar_to_utf8 (0x03b1 + i, p);
+			}
+			*p = '\0';
+			return s;
+		} else {
+			p = s;
+			for (i = 0; i < 32; i++ ) {
+				p += g_unichar_to_utf8 (0xe000 + i, p);
+			}
+			*p = '\0';
+			return s;
+		}
 	} else {
+		/* We have latin encoding */
 		return _("The quick brown fox jumps over the lazy dog.");
 	}
 }
@@ -942,7 +984,7 @@ gnome_font_face_gt1_load (GnomeFontFace * face)
 
 	if (!priv->loadedfont) {
 		g_return_val_if_fail (priv->pfb_fn != NULL, FALSE);
-		priv->loadedfont = gt1_load_font (priv->pfb_fn);
+		priv->loadedfont = gt1_load_font (priv->pfb_fn, priv->privencoding);
 		g_return_val_if_fail (priv->loadedfont != NULL, FALSE);
 	}
 
@@ -952,95 +994,119 @@ gnome_font_face_gt1_load (GnomeFontFace * face)
 #define KERN_PAIR_HASH(g1, g2) ((g1) * 367 + (g2) * 31)
 
 static gboolean
-gff_load_afm (GnomeFontFace *face)
+gff_load_afm (GnomeFontFace * face)
 {
-  Font_Info *fi;
-  FILE *afm_f;
-  int status;
-  gint nglyphs;
+	GnomeFontFacePrivate * priv;
+	Font_Info *fi;
+	FILE *afm_f;
+	int status;
+	gint nglyphs;
+	gint privcode;
 
-  afm_f = fopen (face->private->afm_fn, "r");
-  if (afm_f != NULL)
-    {
-      status = parseFile (afm_f, &fi, P_G | P_M | P_P);
+	priv = face->private;
+
+	afm_f = fopen (face->private->afm_fn, "r");
+	if (afm_f != NULL) {
+		status = parseFile (afm_f, &fi, P_G | P_M | P_P);
 #ifdef VERBOSE
-      debugmsg (_("status loading %s = %d\n"), fontmap_entry->private->afm_fn, status);
+		debugmsg (_("status loading %s = %d\n"), fontmap_entry->private->afm_fn, status);
 #endif
-      if (status == 0)
-	{
-	  GnomeFontLigList **ligtab;
-	  Ligature *ligs;
-	  int i;
-	  GnomeFontKernPair *ktab;
-	  int ktabsize;
+		if (status == 0) {
+			GnomeFontLigList **ligtab;
+			Ligature *ligs;
+			int i;
+			GnomeFontKernPair *ktab;
+			int ktabsize;
 
-	  face->private->ascender = fi->gfi->ascender;
-	  /* This is negated because adobe fonts use a negative number
-             for descenders below the baseline, but most people expect
-             a positive number for descenders below the line. */
-	  face->private->descender = -fi->gfi->descender;
-	  face->private->fixed_width = fi->gfi->isFixedPitch;
+			priv->ascender = fi->gfi->ascender;
+			/* This is negated because adobe fonts use a negative number
+			   for descenders below the baseline, but most people expect
+			   a positive number for descenders below the line. */
+			priv->descender = -fi->gfi->descender;
+			priv->fixed_width = fi->gfi->isFixedPitch;
 
-	  /* Added for pdf */
-	  face->private->capheight = (gdouble) fi->gfi->capHeight;
-	  face->private->italics_angle = (gdouble) fi->gfi->italicAngle;
-	  face->private->xheight = (gdouble) fi->gfi->xHeight;
-	  face->private->bbox.x0 = (gdouble) fi->gfi->fontBBox.llx;
-	  face->private->bbox.y0 = (gdouble) fi->gfi->fontBBox.lly;
-	  face->private->bbox.x1 = (gdouble) fi->gfi->fontBBox.urx;
-	  face->private->bbox.y1 = (gdouble) fi->gfi->fontBBox.ury;
-	  /* */
+			/* Added for pdf */
+			priv->capheight = (gdouble) fi->gfi->capHeight;
+			priv->italics_angle = (gdouble) fi->gfi->italicAngle;
+			priv->xheight = (gdouble) fi->gfi->xHeight;
+			priv->bbox.x0 = (gdouble) fi->gfi->fontBBox.llx;
+			priv->bbox.y0 = (gdouble) fi->gfi->fontBBox.lly;
+			priv->bbox.x1 = (gdouble) fi->gfi->fontBBox.urx;
+			priv->bbox.y1 = (gdouble) fi->gfi->fontBBox.ury;
+			/* */
 	  
-	  face->private->underline_position = fi->gfi->underlinePosition;
-	  face->private->underline_thickness = fi->gfi->underlineThickness;
+			priv->underline_position = fi->gfi->underlinePosition;
+			priv->underline_thickness = fi->gfi->underlineThickness;
 
-	  ligtab = g_malloc (256 * sizeof (GnomeFontLigList *));
-	  face->private->ligs = ligtab;
+			ligtab = g_malloc (256 * sizeof (GnomeFontLigList *));
+			priv->ligs = ligtab;
 
-	  for (i = 0; i < 256; i++)
-	    {
-	      ligtab[i] = 0;
-	    }
+			for (i = 0; i < 256; i++) {
+				ligtab[i] = 0;
+			}
 
-	  face->private->glyphs = g_new0 (GFFGlyphInfo, fi->numOfChars + 1);
-	  face->private->glyphmap = g_hash_table_new (NULL, NULL);
+			priv->glyphs = g_new0 (GFFGlyphInfo, fi->numOfChars + 1);
+			priv->glyphmap = g_hash_table_new (NULL, NULL);
 
-	  gff_fill_zero_glyph (face);
+			gff_fill_zero_glyph (face);
 
-	  nglyphs = 1;
+			nglyphs = 1;
+			privcode = 0xe000;
 
-	  for (i = 0; i < fi->numOfChars; i++) {
-		  CharMetricInfo * info;
-		  gint unicode;
+			for (i = 0; i < fi->numOfChars; i++) {
+				CharMetricInfo * info;
+				gint unicode;
+				gchar * constname;
 
-		  info = fi->cmi + i;
+				constname = NULL; /* Kill warning */
+				info = fi->cmi + i;
 
-		  unicode = gp_unicode_from_ps (info->name);
+				unicode = gp_unicode_from_ps (info->name);
 
-		  if (unicode > 0) {
-			  GFFGlyphInfo * gi;
+				if (unicode < 1) {
+					if (!priv->privencoding) priv->privencoding = g_hash_table_new (g_str_hash, g_str_equal);
+					if (!g_hash_table_lookup (priv->privencoding, info->name)) {
+						unicode = privcode++;
+						constname = g_strdup (info->name);
+						g_hash_table_insert (priv->privencoding, constname, GINT_TO_POINTER (unicode));
+					}
+				} else {
+					constname = (gchar *) gp_const_ps_from_ps (info->name);
+				}
 
-			  gi = face->private->glyphs + nglyphs;
+				if (unicode) {
+					GFFGlyphInfo * gi;
 
-			  gi->unicode = unicode;
-			  gi->psname = gp_const_ps_from_ps (info->name);
+					gi = priv->glyphs + nglyphs;
 
-			  gi->advance.x = info->wx;
-			  gi->advance.y = info->wy;
+					gi->unicode = unicode;
+					gi->psname = constname;
 
-			  gi->bbox.x0 = info->charBBox.llx;
-			  gi->bbox.y0 = info->charBBox.lly;
-			  gi->bbox.x1 = info->charBBox.urx;
-			  gi->bbox.y1 = info->charBBox.ury;
+					gi->advance.x = info->wx;
+					gi->advance.y = info->wy;
 
-			  g_hash_table_insert (face->private->glyphmap, GINT_TO_POINTER (unicode), GINT_TO_POINTER (nglyphs));
+					gi->bbox.x0 = info->charBBox.llx;
+					gi->bbox.y0 = info->charBBox.lly;
+					gi->bbox.x1 = info->charBBox.urx;
+					gi->bbox.y1 = info->charBBox.ury;
 
-			  nglyphs++;
-		  }
-	  }
+					if (gp_multi_from_ps (info->name)) {
+						const GSList * l;
+						l = gp_multi_from_ps (info->name);
+						while (l) {
+							g_hash_table_insert (priv->glyphmap, l->data, GINT_TO_POINTER (nglyphs));
+							l = l->next;
+						}
+					} else {
+						g_hash_table_insert (priv->glyphmap, GINT_TO_POINTER (unicode), GINT_TO_POINTER (nglyphs));
+					}
 
-	  face->private->glyphs = g_renew (GFFGlyphInfo, face->private->glyphs, nglyphs);
-	  face->private->num_glyphs = nglyphs;
+					nglyphs++;
+				}
+			}
+
+			priv->glyphs = g_renew (GFFGlyphInfo, priv->glyphs, nglyphs);
+			priv->num_glyphs = nglyphs;
 
 	  for (i = 0; i < fi->numOfChars; i++)
 	    {
