@@ -65,6 +65,9 @@ struct _GnomeFontFacePrivate {
 	GFFGlyphInfo * glyphs;
 	GHashTable * glyphmap;
 
+	gint num_private;
+	GHashTable * privencoding;
+
 	int   ascender;
 	int   descender;
 	int   underline_position;
