@@ -191,8 +191,8 @@ gint gnome_print_encode_tiff_wcs (gint size)
 /**
  * gnome_print_encode_tiff : Implements TIFF encodeion method for
  *                              PCL printers. Reference : [1]Page15-17.
- *             ( it should be usefull for other Printer Languajes too)
- * @in: the array containing the buffer of bytes tha will be converted
+ *             ( it should be useful for other Printer Languages too)
+ * @in: the array containing the buffer of bytes that will be converted
  * @out: the output array in TIFF "Packbits"
  * @in_size: the size of the input buffer
  * 
@@ -864,18 +864,13 @@ gnome_print_encode_deflate (const guchar *in, guchar *out, gint in_size, gint ou
 	c_stream.avail_in  = in_size;
 	c_stream.avail_out = out_size;
 
-	g_print ("In size %i out %i\n",
-		 in_size, out_size);
-
 	while (c_stream.total_in != (uLong)in_size &&
 	       c_stream.total_out < out_size) {
-		g_print ("loop 1\n");
 		err = deflate(&c_stream, Z_NO_FLUSH);
 		CHECK_ERR(err, "deflate");
 	}
 	/* Finish the stream, still forcing small buffers: */
 	for (;;) {
-		g_print ("Loop 2\n");
 		err = deflate(&c_stream, Z_FINISH);
 		if (err == Z_STREAM_END) break;
 		CHECK_ERR(err, "deflate");
