@@ -35,7 +35,10 @@
  * a GNOME Print Metafile each page.  And we render this
  * metafile into the printing context on page switch.
  */
-#include <config.h>
+/* Must include these two first */
+#include "config.h"
+#include <libgnomeprint/gnome-print-i18n.h>
+
 #include <gnome.h>
 #include <gdk/gdkkeysyms.h>
 #include <libgnomeprint/gnome-printer.h>
@@ -44,6 +47,7 @@
 #include <libgnomeprint/gnome-print-preview.h>
 #include <libgnomeprint/gnome-printer-dialog.h>
 
+#include <libgnomeprint/gnome-print-master-private.h>
 #include "gnome-print-master-preview.h"
 #include "gnome-print-preview-icons.h"
 
@@ -57,6 +61,34 @@ typedef enum {
 	MODE_ZOOM_OUT
 } PreviewMode;
 
+struct _GnomePrintMasterPreview {
+	GnomeApp app;
+
+	void *priv;		/* private data */
+
+	GnomePrintMaster *master; /* what drives us */
+
+	/*
+	 * Preview canvas
+	 */
+	GtkWidget            *scrolled_window;
+	GnomeCanvas	     *canvas;
+
+	/*
+	 * status display
+	 */
+	GtkWidget	    *page_entry;
+	GtkWidget	    *last;
+
+	int                  current_page;
+	int                  pagecount;
+
+};
+
+struct _GnomePrintMasterPreviewClass {
+	GnomeAppClass parent_class;
+};
+
 struct _Private {
 	/* the output print context, a gnomeprintpreview */
 	GnomePrintPreview *output;
@@ -94,7 +126,7 @@ render_page (GnomePrintMasterPreview *pmp, int page)
 	const char *paper_name;
 	struct _Private *pp = pmp->priv;
 
-	gtk_object_destroy (GTK_OBJECT (pp->output));
+	gtk_object_unref (GTK_OBJECT (pp->output));
 	pp->output = NULL;
 
 	/*
@@ -671,6 +703,7 @@ static GnomeUIInfo toolbar [] = {
 static void
 create_toplevel (GnomePrintMasterPreview *pmp)
 {
+	char *old_msg_locale;
 	gint width, height;
 	const GnomePaper *paper;
 	struct _Private *pp;
@@ -700,12 +733,16 @@ create_toplevel (GnomePrintMasterPreview *pmp)
 	gtk_widget_set_usize (GTK_WIDGET (pmp), width, height);
 	gtk_window_set_policy (GTK_WINDOW (pmp), TRUE, TRUE, FALSE);
 
+	old_msg_locale = g_strdup (textdomain (NULL));
+	textdomain (PACKAGE);
 	gnome_app_create_menus_with_data (GNOME_APP(pmp), top_menu, pmp);
 
 	pp->toolbar = g_malloc (sizeof (toolbar));
 	memcpy (pp->toolbar, toolbar, sizeof (toolbar));
 
 	gnome_app_create_toolbar_with_data (GNOME_APP(pmp), pp->toolbar, pmp);
+	textdomain (old_msg_locale);
+	g_free (old_msg_locale);
 }
 
 /*
