@@ -44,11 +44,11 @@ typedef struct _GnomePrintMasterPreviewClass GnomePrintMasterPreviewClass;
 guint		gnome_print_master_preview_get_type	(void);
 
 GnomePrintMasterPreview *
-gnome_print_master_preview_new	(GnomePrintMaster *,
+gnome_print_master_preview_new	(GnomePrintMaster *gpm,
 				 const char *title);
 
 GnomePrintMasterPreview *
-gnome_print_master_preview_new_with_orientation (GnomePrintMaster *,
+gnome_print_master_preview_new_with_orientation (GnomePrintMaster *gpm,
 						 const char *title,
 						 gboolean landscape);
 
