@@ -87,7 +87,6 @@ typedef enum {
 	GNOME_META_CLOSEPATH,
 	GNOME_META_SETRGBCOLOR,
 	GNOME_META_FILL,
-	GNOME_META_EOFILL,
 	GNOME_META_SETLINEWIDTH,
 	GNOME_META_SETMITERLIMIT,
 	GNOME_META_SETLINEJOIN,
@@ -96,15 +95,11 @@ typedef enum {
 	GNOME_META_STROKEPATH,
 	GNOME_META_STROKE,
 	GNOME_META_SETFONT,
-	GNOME_META_SHOW_UCS4,
+	GNOME_META_SHOW_SIZED,
 	GNOME_META_CONCAT,
-#ifdef SETMATRIX_DEPRECATED
-	GNOME_META_SETMATRIX,
-#endif	
 	GNOME_META_GSAVE,
 	GNOME_META_GRESTORE,
 	GNOME_META_CLIP,
-	GNOME_META_EOCLIP,
 	GNOME_META_GRAYIMAGE,
 	GNOME_META_RGBIMAGE,
 	GNOME_META_TEXTLINE,
@@ -124,7 +119,6 @@ static char *meta_type_names[] = {
 	"CLOSEPATH",
 	"SETRGBCOLOR",
 	"FILL",
-	"EOFILL",
 	"SETLINEWIDTH",
 	"SETMITERLIMIT",
 	"SETLINEJOIN",
@@ -133,15 +127,11 @@ static char *meta_type_names[] = {
 	"STROKEPATH",
 	"STROKE",
 	"SETFONT",
-	"SHOW_UCS4",
+	"SHOW_SIZED",
 	"CONCAT",
-#ifdef SETMATRIX_DEPRECATED
-	"SETMATRIX",
-#endif	
 	"GSAVE",
 	"GRESTORE",
 	"CLIP",
-	"EOCLIP",
 	"GRAYIMAGE",
 	"RGBIMAGE",
 	"TEXTLINE",
@@ -296,13 +286,17 @@ encode_six_doubles (GnomePrintContext *pc, double x1, double y1, double x2, doub
 	encode_double (pc, y3);
 }
 
+static void
+encode_string_sized (GnomePrintContext *pc, const char *str, int bytes)
+{
+	encode_int (pc, bytes);
+	encode_block (GNOME_PRINT_META (pc), bytes, str);
+}
+
 static void
 encode_string (GnomePrintContext *pc, const char *str)
 {
-	int len;
-	len = strlen (str);
-	encode_int (pc, len);
-	encode_block (GNOME_PRINT_META (pc), len, str);
+	encode_string_sized (pc, str, strlen (str));
 }
 
 static void
@@ -404,16 +398,10 @@ meta_setrgbcolor (GnomePrintContext *pc, double r, double g, double b)
 }
 
 static int
-meta_fill (GnomePrintContext *pc)
+meta_fill (GnomePrintContext *pc, ArtWindRule rule)
 {
 	encode_int (pc, GNOME_META_FILL);
-	return 0;
-}
-
-static int
-meta_eofill (GnomePrintContext *pc)
-{
-	encode_int (pc, GNOME_META_EOFILL);
+	encode_int (pc, rule);
 	return 0;
 }
 
@@ -500,19 +488,15 @@ meta_setfont (GnomePrintContext *pc, GnomeFont *font)
 
 
 static int
-meta_show_ucs4 (GnomePrintContext *pc, guint32 *buf, gint length)
+meta_show_sized (GnomePrintContext *pc, const char *text, int bytes)
 {
-	gint i;
-
 #ifdef META_DEBUG
-	g_warning ("Show_ucs4 starts at 0x%x",
+	g_warning ("Show_sized starts at 0x%x",
 		   GNOME_PRINT_META (pc)->current);
 #endif
-	encode_int (pc, GNOME_META_SHOW_UCS4);
-	encode_int (pc, length);
-	for (i = 0; i < length; i++) {
-		encode_int (pc, GINT32_FROM_BE (buf[i]));
-	}
+	encode_int (pc, GNOME_META_SHOW_SIZED);
+	encode_string_sized (pc, text, bytes);
+
 	return 0;
 }
 
@@ -524,16 +508,6 @@ meta_concat (GnomePrintContext *pc, const double matrix [6])
 	return 0;
 }
 
-#ifdef SETMATRIX_DEPRECATED
-static int
-meta_setmatrix (GnomePrintContext *pc, const double matrix [6])
-{
-	encode_int (pc, GNOME_META_SETMATRIX);
-	encode_six_doubles (pc, matrix [0], matrix [1], matrix [2], matrix [3], matrix [4], matrix [5]);
-	return 0;
-}
-#endif
-
 static int
 meta_gsave (GnomePrintContext *pc)
 {
@@ -549,16 +523,10 @@ meta_grestore (GnomePrintContext *pc)
 }
 
 static int
-meta_clip (GnomePrintContext *pc)
+meta_clip (GnomePrintContext *pc, ArtWindRule rule)
 {
 	encode_int (pc, GNOME_META_CLIP);
-	return 0;
-}
-
-static int
-meta_eoclip (GnomePrintContext *pc)
-{
-	encode_int (pc, GNOME_META_EOCLIP);
+	encode_int (pc, rule);
 	return 0;
 }
 
@@ -689,7 +657,6 @@ gnome_print_meta_class_init (GnomePrintMetaClass *class)
 	pc_class->closepath = meta_closepath;
 	pc_class->setrgbcolor = meta_setrgbcolor;
 	pc_class->fill = meta_fill;
-	pc_class->eofill = meta_eofill;
 	pc_class->setlinewidth = meta_setlinewidth;
 	pc_class->setmiterlimit = meta_setmiterlimit;
 	pc_class->setlinejoin = meta_setlinejoin;
@@ -698,15 +665,11 @@ gnome_print_meta_class_init (GnomePrintMetaClass *class)
 	pc_class->strokepath = meta_strokepath;
 	pc_class->stroke = meta_stroke;
 	pc_class->setfont = meta_setfont;
-	pc_class->show_ucs4 = meta_show_ucs4;
+	pc_class->show_sized = meta_show_sized;
 	pc_class->concat = meta_concat;
-#ifdef SETMATRIX_DEPRECATED
-	pc_class->setmatrix = meta_setmatrix;
-#endif	
 	pc_class->gsave = meta_gsave;
 	pc_class->grestore = meta_grestore;
 	pc_class->clip = meta_clip;
-	pc_class->eoclip = meta_eoclip;
 	pc_class->grayimage = meta_grayimage;
 	pc_class->rgbimage = meta_rgbimage;
 	pc_class->textline = meta_textline;
@@ -782,7 +745,7 @@ gnome_print_meta_new (void)
 }
 
 /**
- * gnome_print_meta_new:
+ * gnome_print_meta_new_from:
  * @data: Pointer to a gnome_print_meta metadata.
  *
  * Creates a new Metafile context GnomePrint object.
@@ -822,6 +785,7 @@ gnome_print_meta_new_from (const void *data)
 
 /**
  * gnome_print_meta_access_buffer:
+ * @meta:
  * @buffer: points to a void * that will be modified to
  *          point to the new buffer
  * @buflen: pointer to an integer that will be set to the lenght of buffer.
@@ -847,6 +811,7 @@ gnome_print_meta_access_buffer (GnomePrintMeta *meta, void **buffer, int *buflen
 
 /**
  * gnome_print_meta_get_copy:
+ * @meta:
  * @buffer: points to a void * that will be modified to
  *          point to the new buffer
  * @buflen: pointer to an integer that will be set to the lenght of buffer.
@@ -1096,11 +1061,12 @@ do_render (GnomePrintContext *dest, const char *data, int size, int pages)
 			break;
 
 		case GNOME_META_FILL:
-			gnome_print_fill (dest);
-			break;
-
-		case GNOME_META_EOFILL:
-			gnome_print_eofill (dest);
+			data = decode_int (data, &i);
+			if (i == ART_WIND_RULE_NONZERO) {
+				gnome_print_fill (dest);
+			} else {
+				gnome_print_eofill (dest);
+			}
 			break;
 
 		case GNOME_META_SETLINEWIDTH:
@@ -1161,31 +1127,23 @@ do_render (GnomePrintContext *dest, const char *data, int size, int pages)
 			break;
 		}
 
-		case GNOME_META_SHOW_UCS4: {
+		case GNOME_META_SHOW_SIZED: {
 			gint length;
 
 			data = decode_int (data, &length);
 
 			if (length > 0) {
-				guint32 * buf, val;
-				gint i;
+				char *text;
 
-				buf = g_new (guint32, length);
-				for (i = 0; i < length; i++) {
-					data = decode_int (data, &val);
-					buf[i] = GUINT32_TO_BE (val);
-				}
-				gnome_print_show_ucs4 (dest, buf, length);
-				g_free (buf);
+				text = alloca (length);
+				data = decode_block (data, text, length);
+				gnome_print_show_sized (dest, text, length);
 			}
 
 			break;
 		}
 
 		case GNOME_META_CONCAT:
-#ifdef SETMATRIX_DEPRECATED
-		case GNOME_META_SETMATRIX:
-#endif	
 			data = decode_double (data, &matrix [0]);
 			data = decode_double (data, &matrix [1]);
 			data = decode_double (data, &matrix [2]);
@@ -1193,10 +1151,6 @@ do_render (GnomePrintContext *dest, const char *data, int size, int pages)
 			data = decode_double (data, &matrix [4]);
 			data = decode_double (data, &matrix [5]);
 			gnome_print_concat (dest, matrix);
-#ifdef SETMATRIX_DEPRECATED
-			else
-				gnome_print_setmatrix (dest, matrix);
-#endif	
 			break;
 
 		case GNOME_META_GSAVE:
@@ -1208,11 +1162,12 @@ do_render (GnomePrintContext *dest, const char *data, int size, int pages)
 			break;
 
 		case GNOME_META_CLIP:
-			gnome_print_clip (dest);
-			break;
-
-		case GNOME_META_EOCLIP:
-			gnome_print_eoclip (dest);
+			data = decode_int (data, &i);
+			if (i == ART_WIND_RULE_NONZERO) {
+				gnome_print_clip (dest);
+			} else {
+				gnome_print_eoclip (dest);
+			}
 			break;
 
 		case GNOME_META_GRAYIMAGE:
@@ -1303,7 +1258,7 @@ gnome_print_meta_render (GnomePrintContext *destination, const void *meta_stream
 }
 
 /**
- * gnome_print_meta_render_from_object
+ * gnome_print_meta_render_from_object:
  * @destination: Destination printer context.
  * @source: an existing GnomePrintMeta printer context.
  *
@@ -1341,7 +1296,7 @@ gnome_print_meta_render_from_object (GnomePrintContext *destination, const Gnome
 }
 
 /**
- * gnome_print_meta_render:
+ * gnome_print_meta_render_page:
  * @destination: Destination printer context.
  * @meta_stream: a metadata stream to replay
  * @page: Page to be rendered.
