@@ -34,9 +34,7 @@
 */
 
 #define DEBUG
-/*
 #define DITHER_HACK
-*/
 #define RED_VALUE 31
 #define GREEN_VALUE 61
 #define BLUE_VALUE 8
@@ -192,53 +190,19 @@ pclr_print_band (GnomePrintRGBP *rgbp, guchar *rgb_buffer, ArtIRect *rect)
 
 			if ( jobdata->number_of_planes == 1)
 			{
-#ifdef DITHER_HACK				
 				if ( rgb_buffer [offset+j] + rgb_buffer [offset+j+1] + rgb_buffer [offset+j+2] < (255*3) )
 					byte [0] = byte [0] + power_of_2[k-1];
-#else				
-				f =     (255 - rgb_buffer [offset+j]   ) * RED_VALUE +
-					(255 - rgb_buffer [offset+j+1] ) * GREEN_VALUE +
-					(255 - rgb_buffer [offset+j+2] ) * BLUE_VALUE  ;
-				f = f / 255;
-				if (f==0)
-				{
-					zero_count+=1;
-					if (zero_count==2)
-						err[0]=0;
-				}
-				else
-				{
-					zero_count=0;
-				}
-			
-				I = ( f + err[0] + 50 )>99?100:0;
-				err[0] += f - I;
-
-				if (I==100)
-				{
-					byte[0] = byte[0] + power_of_2[k-1];
-				}
-#endif				
 				
 			}
-			else
-			{
-				for (current_plane=0;current_plane < jobdata->number_of_planes; current_plane++)
-					if ( rgb_buffer [offset + j + current_plane ] == 255 )
-						byte [current_plane] = byte [current_plane] + power_of_2[k-1];
-			}
 
 			j+=3;
 			/* every 8 columns write a byte */
 			if ( (j>0) && (j%24 == 0) )
 			{
-				for (current_plane=0;current_plane < jobdata->number_of_planes; current_plane++)
-				{
-					guchar * temp;
-					temp = (guchar *) rowbuff [current_plane];
-					temp [offset_in_row] = byte [current_plane];
-					byte [current_plane] = 0;
-				}
+				guchar * temp;
+				temp = (guchar *) rowbuff [current_plane];
+				temp [offset_in_row] = byte [current_plane];
+				byte [current_plane] = 0;
 				offset_in_row++;
 				k = 0;
 			}
