@@ -13,6 +13,7 @@
 #include <ctype.h>
 
 #include <libart_lgpl/art_affine.h>
+#include <libgnomeprint/gp-unicode.h>
 #include <libgnomeprint/gnome-printer-private.h>
 #include <libgnomeprint/gnome-print-private.h>
 #include <libgnomeprint/gnome-print-ps.h>
@@ -172,17 +173,10 @@ gnome_print_ps_setrgbcolor (GnomePrintContext *pc,
 }
 
 static int
-gnome_print_ps_fill (GnomePrintContext *pc)
+gnome_print_ps_fill (GnomePrintContext *pc, ArtWindRule rule)
 {
   return gnome_print_context_fprintf (pc,
-                                      "fill\n");
-}
-
-static int
-gnome_print_ps_eofill (GnomePrintContext *pc)
-{
-  return gnome_print_context_fprintf (pc,
-                                      "eofill\n");
+				      (rule == ART_WIND_RULE_NONZERO) ? "fill\n" : "eofill\n");
 }
 
 static int
@@ -410,18 +404,18 @@ gnome_print_ps_reencode_font (GnomePrintContext *pc, const char *fontname)
 }
 
 static int
-gnome_print_ps_show_ucs4 (GnomePrintContext *pc, guint32 * buf, gint length)
+gnome_print_ps_show_sized (GnomePrintContext *pc, const char *text, int bytes)
 {
-  int i;
   GnomePrintPs *ps;
-  guint32 u;
+  gunichar u;
+  const char * p;
   ps = GNOME_PRINT_PS (pc);
 
   if (gnome_print_context_fprintf (pc, "(") < 0)
     return -1;
-  for (i = 0; i < length; i++)
+  for (p = text; p && p < (text + bytes); p = g_utf8_next_char (p))
     {
-      u = GUINT32_FROM_BE (buf[i]);
+      u = g_utf8_get_char (p);
       if (u == '(' || u == ')' || u == '\\') 
 	{
 	  if (gnome_print_context_fprintf (pc, "\\%c", u) < 0)
@@ -470,21 +464,6 @@ gnome_print_ps_concat (GnomePrintContext *pc, const double matrix[6])
                                       "%s\n", str);
 }
 
-#ifdef SETMATRIX_DEPRECATED
-static int
-gnome_print_ps_setmatrix (GnomePrintContext *pc, const double matrix[6])
-{
-  return gnome_print_context_fprintf (pc,
-                                      "[ %g %g %g %g %g %g ] setmatrix\n",
-                                      matrix[0],
-                                      matrix[1],
-                                      matrix[2],
-                                      matrix[3],
-                                      matrix[4],
-                                      matrix[5]);
-}
-#endif
-
 static int
 gnome_print_ps_gsave (GnomePrintContext *pc)
 {
@@ -500,17 +479,10 @@ gnome_print_ps_grestore (GnomePrintContext *pc)
 }
 
 static int
-gnome_print_ps_clip (GnomePrintContext *pc)
+gnome_print_ps_clip (GnomePrintContext *pc, ArtWindRule rule)
 {
   return gnome_print_context_fprintf (pc,
-                                      "clip\n");
-}
-
-static int
-gnome_print_ps_eoclip (GnomePrintContext *pc)
-{
-  return gnome_print_context_fprintf (pc,
-                                      "eoclip\n");
+                                      (rule == ART_WIND_RULE_NONZERO) ? "clip\n" : "eoclip\n");
 }
 
 static int
@@ -737,7 +709,6 @@ gnome_print_ps_class_init (GnomePrintPsClass *class)
   pc_class->closepath = gnome_print_ps_closepath;
   pc_class->setrgbcolor = gnome_print_ps_setrgbcolor;
   pc_class->fill = gnome_print_ps_fill;
-  pc_class->eofill = gnome_print_ps_eofill;
   pc_class->setlinewidth = gnome_print_ps_setlinewidth;
   pc_class->setmiterlimit = gnome_print_ps_setmiterlimit;
   pc_class->setlinejoin = gnome_print_ps_setlinejoin;
@@ -746,15 +717,11 @@ gnome_print_ps_class_init (GnomePrintPsClass *class)
   pc_class->strokepath = gnome_print_ps_strokepath;
   pc_class->stroke = gnome_print_ps_stroke;
   pc_class->setfont = gnome_print_ps_setfont;
-  pc_class->show_ucs4 = gnome_print_ps_show_ucs4;
+  pc_class->show_sized = gnome_print_ps_show_sized;
   pc_class->concat = gnome_print_ps_concat;
-#ifdef SETMATRIX_DEPRECATED
-  pc_class->setmatrix = gnome_print_ps_setmatrix;
-#endif	
   pc_class->gsave = gnome_print_ps_gsave;
   pc_class->grestore = gnome_print_ps_grestore;
   pc_class->clip = gnome_print_ps_clip;
-  pc_class->eoclip = gnome_print_ps_eoclip;
   pc_class->grayimage = gnome_print_ps_grayimage;
   pc_class->rgbimage = gnome_print_ps_rgbimage;
   pc_class->textline = gnome_print_ps_textline;
