@@ -40,6 +40,7 @@
 
 #include <libart_lgpl/art_affine.h>
 #include <libart_lgpl/art_misc.h>
+#include <libgnomeprint/gp-unicode.h>
 #include <libgnomeprint/gnome-print-private.h>
 #include <libgnomeprint/gnome-printer-private.h>
 #include <libgnomeprint/gnome-print-ps2.h>
@@ -53,7 +54,7 @@
 #define GNOME_PRINT_PS2_BUFFER_SIZE 1024
 #define GNOME_PRINT_PS2_NUMBER_OF_ELEMENTS 3
 #define GNOME_PRINT_PS2_NUMBER_OF_ELEMENTS_GROW 2
-#define GNOME_PRINT_PS2_FONT_UNDEFINED 999
+#define GNOME_PRINT_PS2_FONT_UNDEFINED 9999
 
 static GnomePrintContextClass *parent_class = NULL;
 
@@ -177,7 +178,7 @@ struct _GnomePrintPs2Class
 
 static void gnome_print_ps2_class_init    (GnomePrintPs2Class *klass);
 static void gnome_print_ps2_init          (GnomePrintPs2 *PS2);
-static void gnome_print_ps2_finalize      (GtkObject *object);
+static void gnome_print_ps2_destroy       (GtkObject *object);
 static gint gnome_print_ps2_dictionary    (GnomePrintContext *pc);
 
 /* -------------------------- PUBLIC FUNCTIONS ------------------------ */
@@ -246,6 +247,8 @@ gnome_print_ps2_new (GnomePrinter *printer)
 																		 "Document Name Goes Here",
 																		 "Author Goes Here",
 																		 date);
+
+	g_free (date);
 	
 	if ( ret < 0)
 		goto failure;
@@ -497,7 +500,7 @@ gnome_print_ps2_showpage (GnomePrintContext *pc)
 	g_return_val_if_fail (ps2->current_page != NULL, -1);
 
 	if (ps2->current_page->showpaged) {
-		gnome_print_ps2_error (TRUE, "showpage, showpaged used twice for the same page");
+		gnome_print_ps2_error (TRUE , "showpage, showpaged used twice for the same page");
 		return 0;
 	}
 	
@@ -505,14 +508,17 @@ gnome_print_ps2_showpage (GnomePrintContext *pc)
 		gnome_print_ps2_error (FALSE, "showpage, with graphic state stack NOT empty");
 
 	ps2->current_page->showpaged = TRUE;
-	
+
 	gnome_print_ps2_page_end   (pc);
 
-	/* We need to clean the Graphic State */
+	/* We need to clean the Graphic State. Chema */
+#if 0	/* I don't see the need for cleaning the state, this is actually
+				 wrong, since postscript does not function that way */
 	gnome_print_ps2_graphic_state_free (ps2->graphic_state);
 	gnome_print_ps2_graphic_state_free (ps2->graphic_state_set);
 	ps2->graphic_state     = gnome_print_ps2_graphic_state_new (FALSE);
 	ps2->graphic_state_set = gnome_print_ps2_graphic_state_new (TRUE);
+#endif	
 	
 	gnome_print_ps2_page_start (pc);
 
@@ -525,7 +531,7 @@ gnome_print_ps2_beginpage (GnomePrintContext *pc, const char *name_of_this_page)
 	GnomePrintPs2 *ps2;
 
 	debug (FALSE, "");
-	
+
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
 	g_return_val_if_fail (name_of_this_page != NULL, -1);
 	ps2 = GNOME_PRINT_PS2 (pc);
@@ -535,7 +541,9 @@ gnome_print_ps2_beginpage (GnomePrintContext *pc, const char *name_of_this_page)
 	
 	ps2->current_page->page_name = g_strdup (name_of_this_page);
 
-	return 0;
+	return gnome_print_context_fprintf (pc,
+                                      "%%%%Page: %s\n",
+																			name_of_this_page);
 }
 
 
@@ -851,28 +859,8 @@ gnome_print_ps2_graphic_state_text_set (GnomePrintContext *pc)
 	return gs;
 }
 
-#if 0
-static void
-gnome_print_ps2_reencode_font (GnomePrintContext *pc, GnomePrintPs2Font *font)
-{
-	g_return_if_fail (GNOME_IS_PRINT_CONTEXT (pc));
-	g_return_if_fail (font != NULL);
-	g_return_if_fail (font->reencoded == FALSE);
-
-	/* TODO : define the reencode procedure and just call
-		 as "/Fontname RE". Chema */
-  gnome_print_context_fprintf (pc,
-															 "/%s FF RE" EOL
-															 "/La-%s EX DF p" EOL,
-															 font->font_name, font->font_name);
-
-	font->reencoded = TRUE;
-}
-#endif
-
 #define gpcf gnome_print_context_fprintf
-
-static void
+static gboolean
 gnome_print_ps2_reencode_font (GnomePrintContext *pc, GnomePrintPs2Font *ps2font)
 {
 	GnomeFont * font;
@@ -880,10 +868,16 @@ gnome_print_ps2_reencode_font (GnomePrintContext *pc, GnomePrintPs2Font *ps2font
 	gint nglyphs, nfonts;
 	gint i, j;
 
+	debug (FALSE, "");
+	
 	/* Bitch 'o' bitches (Lauris) ! */
 
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), FALSE);
 	font = ps2font->gnome_font;
+	g_return_val_if_fail (GNOME_IS_FONT (font), FALSE);
 	face = (GnomeFontFace *) gnome_font_get_face (font);
+	g_return_val_if_fail (GNOME_IS_FONT_FACE (face), FALSE);
+		
 	nglyphs = gnome_font_face_get_num_glyphs (face);
 	nfonts = (nglyphs + 255) >> 8;
 
@@ -914,9 +908,10 @@ gnome_print_ps2_reencode_font (GnomePrintContext *pc, GnomePrintPs2Font *ps2font
 			gint glyph;
 
 			glyph = 256 * i + j;
-			if (glyph >= nglyphs) glyph = 0;
+			if (glyph >= nglyphs)
+				glyph = 0;
 
-			gpcf (pc, "/%s" EOL, gnome_font_face_get_glyph_ps_name (face, glyph));
+			gpcf (pc, ((j & 0x0f) == 0x0f) ? "/%s" EOL : "/%s ", gnome_font_face_get_glyph_ps_name (face, glyph));
 		}
 
 		gpcf (pc, "] def" EOL);
@@ -933,7 +928,7 @@ gnome_print_ps2_reencode_font (GnomePrintContext *pc, GnomePrintPs2Font *ps2font
 
 		fn = (i < nfonts) ? i : 0;
 
-		gpcf (pc, "%d " EOL, fn);
+		gpcf (pc, ((i & 0x0f) == 0x0f) ? "%d" EOL : "%d  ", fn);
 	}
 
 	gpcf (pc, "] def" EOL);
@@ -942,6 +937,8 @@ gnome_print_ps2_reencode_font (GnomePrintContext *pc, GnomePrintPs2Font *ps2font
 	gpcf (pc, "/Uni-%s EX DF p" EOL, gnome_font_face_get_ps_name (face));
 
 	ps2font->reencoded = TRUE;
+
+	return TRUE;
 }
 
 #undef gpcf
@@ -950,7 +947,9 @@ static void
 gnome_print_ps2_download_font (GnomePrintContext *pc, GnomePrintPs2Font *font)
 {
 	gchar *pfa;
-	
+
+	debug (FALSE, "");
+		
 	g_return_if_fail (GNOME_IS_PRINT_CONTEXT (pc));
 	g_return_if_fail (font != NULL);
 	g_return_if_fail (font->reencoded == FALSE);
@@ -989,9 +988,23 @@ gnome_print_ps2_graphic_state_setfont (GnomePrintContext *pc)
 	g_return_val_if_fail (ps2->graphic_state->ps2_font_number !=
 												GNOME_PRINT_PS2_FONT_UNDEFINED, -1);
 
+#if 0	
+	g_print ("Font number in state %i, font number set %i\n",
+					 gs->ps2_font_number, gs_set->ps2_font_number);
+#endif	
+					 
 	ps2_font     = &ps2->fonts [gs->ps2_font_number];
 	ps2_font_set = &ps2->fonts [gs_set->ps2_font_number];
 
+#if 0	
+	g_print ("Address of font is %i,%i (num:%i)\n",
+					 GPOINTER_TO_INT (ps2_font),
+					 GPOINTER_TO_INT (ps2_font->gnome_font),
+					 gs->ps2_font_number);
+#endif	
+					 
+	g_return_val_if_fail (GNOME_IS_FONT (ps2_font->gnome_font),-1);
+	
 	if ((ps2_font == ps2_font_set) &&
 			(gs->font_size == gs_set->font_size))
 		return ret;
@@ -1009,7 +1022,7 @@ gnome_print_ps2_graphic_state_setfont (GnomePrintContext *pc)
 																			gs->font_size);
 	gs_set->font_size = gs->font_size;
 	gs_set->ps2_font_number = gs->ps2_font_number;
-	
+
 	return ret;
 }
 
@@ -1400,6 +1413,7 @@ gnome_print_ps2_get_font_number (GnomePrintContext *pc, GnomeFont *gnome_font,
 	debug (FALSE, "");
 
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (GNOME_IS_FONT (gnome_font), -1);
 	ps2 = GNOME_PRINT_PS2 (pc);
 	g_return_val_if_fail (ps2 != NULL, -1);
 	
@@ -1414,13 +1428,16 @@ gnome_print_ps2_get_font_number (GnomePrintContext *pc, GnomeFont *gnome_font,
 		return n;
 	}
 
-	if (ps2->fonts_number == ps2->fonts_max)
+	if (ps2->fonts_number == ps2->fonts_max) {
 		ps2->fonts = g_realloc (ps2->fonts, sizeof (GnomePrintPs2Font) *
 														(ps2->fonts_max += GNOME_PRINT_PS2_NUMBER_OF_ELEMENTS_GROW));
+	}
 
 	font = &ps2->fonts[ps2->fonts_number++];
 	font->font_number   = ps2->fonts_number*2;
+	gnome_font_ref (gnome_font);
 	font->gnome_font    = gnome_font;
+	
 	font->font_name     = font_name;
 	font->reencoded     = FALSE;
 	font->external      = external;
@@ -1489,12 +1506,14 @@ gnome_print_ps2_setfont (GnomePrintContext *pc, GnomeFont *font)
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
   ps2 = GNOME_PRINT_PS2 (pc);
 	g_return_val_if_fail (ps2 != NULL, -1);
-
-  if (font == NULL)
-    return -1;
+	g_return_val_if_fail (GNOME_IS_FONT (font), -1);
 
   fontname = gnome_font_get_ps_name (font);
 
+#if 0	
+	g_print ("Setting font [%s,%i]\n", fontname, GPOINTER_TO_INT (font));
+#endif	
+
   for (n = 0; n < ps2->fonts_internal_number; n++)
     if (!strcmp (fontname, ps2->fonts_internal[n]))
       break;
@@ -1531,21 +1550,24 @@ gnome_print_ps2_setfont (GnomePrintContext *pc, GnomeFont *font)
 }
 
 static gint
-gnome_print_ps2_show_ucs4 (GnomePrintContext *pc, guint32 * buf, gint length)
+gnome_print_ps2_show_sized (GnomePrintContext *pc, const char *text, int bytes)
 {
 	GnomePrintPs2 *ps2;
 	GnomePrintPs2GraphicState *gs;
 	ArtPoint point;
 	gint ret = 0;
-	gint i;
+	const char *p;
 	const GnomeFontFace * face;
 
 	debug (FALSE, "");
 
 	ps2 = (GnomePrintPs2 *) pc;
 
-	gs = ps2->graphic_state;
+	g_return_val_if_fail (GNOME_IS_PRINT_PS2 (ps2), -1);
+	g_return_val_if_fail (ps2->fonts != NULL, -1);
 
+	gs = ps2->graphic_state;
+	
 	if (!gp_path_has_currentpoint (gs->current_path)) {
 		gnome_print_ps2_error (FALSE, "show, currentpoint not defined.");
 		return -1;
@@ -1561,8 +1583,12 @@ gnome_print_ps2_show_ucs4 (GnomePrintContext *pc, guint32 * buf, gint length)
 
 	ret += gnome_print_ps2_graphic_state_setfont (pc);
 
+	g_return_val_if_fail (GNOME_IS_FONT (ps2->fonts[gs->ps2_font_number].gnome_font), -1);
+
 	face = gnome_font_get_face (ps2->fonts[gs->ps2_font_number].gnome_font);
 
+	g_return_val_if_fail (GNOME_IS_FONT_FACE (face), -1);
+
 	ret += gnome_print_context_fprintf (pc,
 																			"%g %g Tm" EOL,
 																			point.x,
@@ -1575,17 +1601,18 @@ gnome_print_ps2_show_ucs4 (GnomePrintContext *pc, guint32 * buf, gint length)
   if (gnome_print_context_fprintf (pc, "(") < 0)
     return -1;
 
-  for (i = 0; i < length; i++)
+  for (p = text; p && p < (text + bytes); p = g_utf8_next_char (p))
 	{
-		guint32 u;
+		gunichar u;
 		gint g, glyph, page;
 
-    u = GUINT32_FROM_BE (buf[i]);
+    u = g_utf8_get_char (p);
 		g = gnome_font_face_lookup_default (face, u);
 		glyph = g & 0xff;
 		page = (g >> 8) & 0xff;
 
-		if (gnome_print_context_fprintf (pc,"\\%o\\%o", page, glyph) < 0) return -1;
+		if (gnome_print_context_fprintf (pc,"\\%03o\\%03o", page, glyph) < 0)
+			return -1;
 	}
 
 	ret += gnome_print_context_fprintf (pc, ") Tj" EOL);
@@ -1675,39 +1702,7 @@ gnome_print_ps2_glyphlist (GnomePrintContext *pc, GnomeGlyphList *gl)
 }
 
 static gint
-gnome_print_ps2_eoclip (GnomePrintContext *pc)
-{
-	GnomePrintPs2 *ps2;
-	GnomePrintPs2GraphicState *gs;
-	gint ret = 0;
-	
-	debug (FALSE, "");
-
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
-	ps2 = GNOME_PRINT_PS2 (pc);
-	g_return_val_if_fail (ps2 != NULL, -1);
-
-	gs = gnome_print_ps2_graphic_state_set (pc);
-
-#ifdef FIND_A_WAY_TO_CHECK_FOR_THIS		
-	if (gp_path_length (gs->current_path) < 2) {
-		gnome_print_ps2_error (FALSE, "Trying to clip with an empty path. (EO)");
-		gp_path_reset (gs->current_path);
-		return -1;
-	}
-#endif
-	
-	ret += gnome_print_ps2_path_print  (pc, gs->current_path);
-	ret += gnome_print_context_fprintf (pc, "W* n" EOL);
-
-	gp_path_reset (gs->current_path);
-		
-  return ret;
-}
-
-
-static gint
-gnome_print_ps2_clip (GnomePrintContext *pc)
+gnome_print_ps2_clip (GnomePrintContext *pc, ArtWindRule rule)
 {
 	GnomePrintPs2 *ps2;
 	GnomePrintPs2GraphicState *gs;
@@ -1731,7 +1726,11 @@ gnome_print_ps2_clip (GnomePrintContext *pc)
 #endif	
 
 	ret += gnome_print_ps2_path_print  (pc, gs->current_path);
-	ret += gnome_print_context_fprintf (pc, "W n" EOL);
+	if (rule == ART_WIND_RULE_NONZERO) {
+		ret += gnome_print_context_fprintf (pc, "W n" EOL);
+	} else {
+		ret += gnome_print_context_fprintf (pc, "W n" EOL);
+	}
 
 	gp_path_reset (gs->current_path);
 		
@@ -1771,39 +1770,7 @@ gnome_print_ps2_stroke (GnomePrintContext *pc)
 }
 
 static gint
-gnome_print_ps2_eofill (GnomePrintContext *pc)
-{
-	GnomePrintPs2 *ps2;
-	GnomePrintPs2GraphicState *gs;
-	gint ret = 0;
-
-	debug (FALSE, "");
-
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
-	ps2 = GNOME_PRINT_PS2 (pc);
-	g_return_val_if_fail (ps2 != NULL, -1);
-
-	gs = gnome_print_ps2_graphic_state_set (pc);
-
-#ifdef FIND_A_WAY_TO_CHECK_FOR_THIS		
-	if (gp_path_length (gs->current_path) < 2) {
-		gnome_print_ps2_error (FALSE, "Trying to fill an empty path");
-#if 0	
-		gp_path_reset (gs->current_path);
-		return -1;
-#endif	
-	}
-#endif	
-
-	ret += gnome_print_ps2_path_print  (pc, gs->current_path);
-	ret += gnome_print_context_fprintf (pc, "f*" EOL);
-
-  return ret;
-}
-
-
-static gint
-gnome_print_ps2_fill (GnomePrintContext *pc)
+gnome_print_ps2_fill (GnomePrintContext *pc, ArtWindRule rule)
 {
 	GnomePrintPs2 *ps2;
 	GnomePrintPs2GraphicState *gs;
@@ -1828,7 +1795,11 @@ gnome_print_ps2_fill (GnomePrintContext *pc)
 #endif
 	
 	ret += gnome_print_ps2_path_print  (pc, gs->current_path);
-	ret += gnome_print_context_fprintf (pc, "f" EOL);
+	if (rule == ART_WIND_RULE_NONZERO) {
+		ret += gnome_print_context_fprintf (pc, "f" EOL);
+	} else {
+		ret += gnome_print_context_fprintf (pc, "f*" EOL);
+	}
 
   return ret;
 }
@@ -2104,24 +2075,6 @@ gnome_print_ps2_textline (GnomePrintContext *pc, GnomeTextLine *line)
 	return 0;
 }
 
-#ifdef SETMATRIX_DEPRECATED
-static gint
-gnome_print_ps2_setmatrix (GnomePrintContext *pc, const double matrix[6])
-{
-	static gint warned = FALSE;
-
-	debug (FALSE, "");
-
-	if (warned)
-		return 0;
-
-	g_warning ("Apps should not call setmatrix !!!. This function will be deprecated");
-	warned = TRUE;
-	
-	return 0;
-}
-#endif
-
 #define G_P_VERY_SMALL (double) 1e-9
 static void
 gnome_print_ps2_matrix_clean (double matrix[6])
@@ -2446,13 +2399,27 @@ static char *gnome_print_ps2_internal_fonts[] = {
 };
 
 
-static void
-gnome_print_ps2_finalize (GtkObject *object)
+static gint
+gnome_print_ps2_page_free (GnomePrintPs2Page *page)
 {
+	g_return_val_if_fail (page != NULL, FALSE);
+
+	if (page->page_name)
+		g_free (page->page_name);
+	g_free (page);
+
+	return TRUE;
+}
+
+static void
+gnome_print_ps2_destroy (GtkObject *object)
+{
+	GnomePrintPs2Font *font;
 	GnomePrintPs2Page *page;
 	GnomePrintPs2 *ps2;
 	GList *list;
 	gint showpaged_all;
+	gint n;
 
 	debug (FALSE, "");
 
@@ -2460,9 +2427,12 @@ gnome_print_ps2_finalize (GtkObject *object)
   g_return_if_fail (GNOME_IS_PRINT_PS2 (object));
   ps2 = GNOME_PRINT_PS2 (object);
 
-	/* gsave stack */
+	/* gsave stack & graphic States*/
 	if (ps2->gsave_level_number != 0)
 		g_warning ("gsave unmatched. Should end with an empty stack");
+	g_free (ps2->gsave_stack);
+	gnome_print_ps2_graphic_state_free (ps2->graphic_state);
+	gnome_print_ps2_graphic_state_free (ps2->graphic_state_set);
 
 	if (g_list_length (ps2->pages) == 0)
 		showpaged_all = FALSE;
@@ -2474,9 +2444,10 @@ gnome_print_ps2_finalize (GtkObject *object)
 		page = (GnomePrintPs2Page *) list->data;
 		if (!page->showpaged)
 			showpaged_all = FALSE;
-		g_free (page->page_name);
-		g_free (page);
+		gnome_print_ps2_page_free (page);
 	}
+	gnome_print_ps2_page_free (ps2->current_page);
+	g_list_free (ps2->pages);
 
 	if (!showpaged_all)
 		g_warning ("The application didn't called \"showpage\" for\n"
@@ -2486,14 +2457,19 @@ gnome_print_ps2_finalize (GtkObject *object)
 							 "THIS IS NOT A GNOME-PRINT BUG !!\n\n"
 							 "Love all, serve all\n");
 
-	g_list_free (ps2->pages);
+
+	for (n=0; n < ps2->fonts_number; n++) {
+		font = &ps2->fonts[n];
+		gtk_object_unref (GTK_OBJECT (font->gnome_font));
+		g_free (font->font_name);
+	}
 	
-	g_free (ps2->gsave_stack);
 	g_free (ps2->fonts); 
 	g_free (ps2->fonts_internal);
+	if (ps2->fonts_external) g_free (ps2->fonts_external);
 
-  (* GTK_OBJECT_CLASS (parent_class)->finalize) (object);
-	
+  if (* GTK_OBJECT_CLASS (parent_class)->destroy)
+		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
 
 static void
@@ -2501,7 +2477,7 @@ gnome_print_ps2_init (GnomePrintPs2 *ps2)
 {
 	GnomePrintContext *pc = GNOME_PRINT_CONTEXT (ps2);
 	gint n;
-	
+
 	debug (FALSE, "");
 
 	ps2->current_page = NULL;
@@ -2549,7 +2525,7 @@ gnome_print_ps2_class_init (GnomePrintPs2Class *class)
 
 	parent_class = gtk_type_class (gnome_print_context_get_type ());
 	
-  object_class->finalize = gnome_print_ps2_finalize;
+  object_class->destroy = gnome_print_ps2_destroy;
 
   pc_class->newpath         = gnome_print_ps2_newpath;
   pc_class->moveto          = gnome_print_ps2_moveto;
@@ -2558,7 +2534,6 @@ gnome_print_ps2_class_init (GnomePrintPs2Class *class)
   pc_class->closepath       = gnome_print_ps2_closepath;
   pc_class->setrgbcolor     = gnome_print_ps2_setrgbcolor;
   pc_class->fill            = gnome_print_ps2_fill;
-  pc_class->eofill          = gnome_print_ps2_eofill;
   pc_class->setlinewidth    = gnome_print_ps2_setlinewidth;
   pc_class->setmiterlimit   = gnome_print_ps2_setmiterlimit;
   pc_class->setlinejoin     = gnome_print_ps2_setlinejoin;
@@ -2567,15 +2542,11 @@ gnome_print_ps2_class_init (GnomePrintPs2Class *class)
   pc_class->strokepath      = gnome_print_ps2_strokepath;
   pc_class->stroke          = gnome_print_ps2_stroke;
   pc_class->setfont         = gnome_print_ps2_setfont;
-  pc_class->show_ucs4       = gnome_print_ps2_show_ucs4;
+  pc_class->show_sized      = gnome_print_ps2_show_sized;
   pc_class->concat          = gnome_print_ps2_concat;
-#ifdef SETMATRIX_DEPRECATED
-  pc_class->setmatrix       = gnome_print_ps2_setmatrix;
-#endif		
   pc_class->gsave           = gnome_print_ps2_gsave;
   pc_class->grestore        = gnome_print_ps2_grestore;
   pc_class->clip            = gnome_print_ps2_clip;
-  pc_class->eoclip          = gnome_print_ps2_eoclip;
   pc_class->grayimage       = gnome_print_ps2_grayimage;
   pc_class->rgbimage        = gnome_print_ps2_rgbimage;
   pc_class->textline        = gnome_print_ps2_textline;
