@@ -7,13 +7,25 @@
  *   Miguel de Icaza (miguel@kernel.org)
  *   Lauris Kaplinski <lauris@ariman.ee>
  */
+
+/*
+ * GnomePrintContext now has graphicContext member. It is updated by every
+ * method, except clip and eoclip - I yet have to figure out, whether maintaining
+ * explicit SVP-s here is worth the effort.
+ * GC is updated AFTER corresponding class method is invoked - we expect, that if
+ * context implements class method, it manages it's own data.
+ * So you do not have to implement most class methods any more - simply wait for
+ * interesting ones (fill, stroke), and update current state from GC.
+ * GC is also used to check errors, so lineto without preceding moveto is now
+ * handled by GnomePrintContext itself. -Lauris-
+ */
+
 /* Must include these two first */
 #include "config.h"
 #include <libgnomeprint/gnome-print-i18n.h>
 
 #include <string.h>
 #include <locale.h>
-#include <unicode.h>
 
 #include <gtk/gtk.h>
 #include <stdio.h>
@@ -23,11 +35,12 @@
 /* gnome.h is only needed for g_concat_string_and_file */
 #include <gnome.h>
 
+#include <libgnomeprint/gp-unicode.h>
 #include <libgnomeprint/gnome-printer-private.h>
 #include <libgnomeprint/gnome-print.h>
 #include <libgnomeprint/gnome-print-private.h>
 
-/* Gnome Print Drivres */
+/* Gnome Print Drivers */
 #include <libgnomeprint/gnome-print-ps.h>
 #include <libgnomeprint/gnome-print-ps2.h>
 #include <libgnomeprint/gnome-print-pdf.h>
@@ -85,47 +98,13 @@ gnome_print_context_class_init (GnomePrintContextClass *class)
 	parent_class = gtk_type_class (gtk_object_get_type ());
 
 	object_class->finalize = gnome_print_context_finalize;
-#if 0
-	/* Well - according to spec GtkObjectClass if zeroed */
-	class->newpath = NULL;
-	class->moveto = NULL;
-	class->lineto = NULL;
-	class->curveto = NULL;
-	class->closepath = NULL;
-	class->setrgbcolor = NULL;
-	class->fill = NULL;
-	class->eofill = NULL;
-	class->setlinewidth = NULL;
-	class->setmiterlimit = NULL;
-	class->setlinejoin = NULL;
-	class->setlinecap = NULL;
-	class->setdash = NULL;
-	class->strokepath = NULL;
-	class->stroke = NULL;
-	class->setfont = NULL;
-	class->show_ucs4 = NULL;
-	class->concat = NULL;
-#ifdef SETMATRIX_DEPRECATED
-	class->setmatrix = NULL;
-#endif
-	class->gsave = NULL;
-	class->grestore = NULL;
-	class->clip = NULL;
-	class->eoclip = NULL;
-	class->grayimage = NULL;
-	class->rgbimage = NULL;
-	class->textline = NULL;
-	class->beginpage = NULL;
-	class->showpage = NULL;
-	class->close = NULL;
-	class->setopacity = NULL;
-	class->rgbaimage = NULL;
-#endif
 }
 
 static void
 gnome_print_context_init (GnomePrintContext *pc)
 {
+	pc->gc = gp_gc_new ();
+
 	pc->f = NULL;
 }
 
@@ -141,6 +120,7 @@ gnome_print_context_new_with_paper_size (GnomePrinter *printer, const char *pape
 	if (strcmp (printer->driver, "gnome-print-ps") == 0) {
 		GnomePrintPs *ps = gnome_print_ps_new (printer);
 		GnomePrintContext *frgba = gnome_print_frgba_new ((GnomePrintContext *) ps);
+		gtk_object_unref (GTK_OBJECT (ps));
 		return frgba ? frgba : NULL;
 	}
 	if (strcmp (printer->driver, "gnome-print-ps-rgb") == 0) {
@@ -150,10 +130,11 @@ gnome_print_context_new_with_paper_size (GnomePrinter *printer, const char *pape
 	if (strcmp (printer->driver, "gnome-print-ps2") == 0) {
 		GnomePrintPs2 *ps2 = gnome_print_ps2_new (printer);
 		GnomePrintContext *frgba = gnome_print_frgba_new ((GnomePrintContext *) ps2);
+		gtk_object_unref (GTK_OBJECT (ps2));
 		return frgba ? frgba : NULL;
 	}
 	if (strcmp (printer->driver, "gnome-print-pdf") == 0) {
-		GnomePrintPdf *pdf = gnome_print_pdf_new (printer);
+		GnomePrintPdf *pdf = gnome_print_pdf_new_with_paper (printer, paper_size);
 		return pdf ? GNOME_PRINT_CONTEXT (pdf) : NULL;
 	}
 
@@ -193,11 +174,10 @@ gnome_print_context_finalize (GtkObject *object)
 {
 	GnomePrintContext *pc;
 
-	g_return_if_fail (object != NULL);
-	g_return_if_fail (GNOME_IS_PRINT_CONTEXT (object));
-
 	pc = GNOME_PRINT_CONTEXT (object);
 
+	gp_gc_unref (pc->gc);
+
 	(* GTK_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 
@@ -208,301 +188,424 @@ gnome_print_context_finalize (GtkObject *object)
 int
 gnome_print_newpath (GnomePrintContext *pc)
 {
+	gint ret = GNOME_PRINT_OK;
+
 	g_return_val_if_fail (pc != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
 
-	return PRINT_CLASS(pc)->newpath (pc);
+	if (PRINT_CLASS (pc)->newpath)
+		ret =  PRINT_CLASS(pc)->newpath (pc);
+
+	gp_gc_newpath (pc->gc);
+
+	return ret;
 }
 
 int
 gnome_print_moveto (GnomePrintContext *pc, double x, double y)
 {
+	gint ret = GNOME_PRINT_OK;
+
 	g_return_val_if_fail (pc != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
 
-	return PRINT_CLASS(pc)->moveto (pc, x, y);
+	if (PRINT_CLASS (pc)->moveto)
+		ret = PRINT_CLASS(pc)->moveto (pc, x, y);
+
+	gp_gc_moveto (pc->gc, x, y);
+
+	return ret;
 }
 
 int
 gnome_print_lineto (GnomePrintContext *pc, double x, double y)
 {
+	gint ret = GNOME_PRINT_OK;
+
 	g_return_val_if_fail (pc != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (gp_gc_has_currentpoint (pc->gc), GNOME_PRINT_ERROR_NOCURRENTPOINT);
 
-	return PRINT_CLASS(pc)->lineto (pc, x, y);
+	if (PRINT_CLASS (pc)->lineto)
+		ret = PRINT_CLASS(pc)->lineto (pc, x, y);
+
+	gp_gc_lineto (pc->gc, x, y);
+
+	return ret;
 }
 
 int
 gnome_print_curveto (GnomePrintContext *pc, double x1, double y1, double x2, double y2, double x3, double y3)
 {
+	gint ret = GNOME_PRINT_OK;
+
 	g_return_val_if_fail (pc != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (gp_gc_has_currentpoint (pc->gc), GNOME_PRINT_ERROR_NOCURRENTPOINT);
 
-	return PRINT_CLASS(pc)->curveto (pc, x1, y1, x2, y2, x3, y3);
+	if (PRINT_CLASS (pc)->curveto)
+		ret = PRINT_CLASS(pc)->curveto (pc, x1, y1, x2, y2, x3, y3);
+
+	gp_gc_curveto (pc->gc, x1, y1, x2, y2, x3, y3);
+
+	return ret;
 }
 
 int
 gnome_print_closepath (GnomePrintContext *pc)
 {
+	gint ret = GNOME_PRINT_OK;
+
 	g_return_val_if_fail (pc != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (gp_gc_has_currentpath (pc->gc), GNOME_PRINT_ERROR_NOCURRENTPATH);
 
-	return PRINT_CLASS(pc)->closepath (pc);
+	if (PRINT_CLASS (pc)->closepath)
+		ret = PRINT_CLASS(pc)->closepath (pc);
+
+	gp_gc_closepath (pc->gc);
+
+	return ret;
 }
 
 int
 gnome_print_setrgbcolor (GnomePrintContext *pc, double r, double g, double b)
 {
+	gint ret = GNOME_PRINT_OK;
+
 	g_return_val_if_fail (pc != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
 
-	return PRINT_CLASS(pc)->setrgbcolor (pc, r, g, b);
+	if (PRINT_CLASS (pc)->setrgbcolor)
+		ret = PRINT_CLASS(pc)->setrgbcolor (pc, r, g, b);
+
+	gp_gc_set_rgbcolor (pc->gc, r, g, b);
+
+	return ret;
 }
 
 int
 gnome_print_fill (GnomePrintContext *pc)
 {
+	gint ret = GNOME_PRINT_OK;
+
 	g_return_val_if_fail (pc != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (gp_gc_has_currentpath (pc->gc), GNOME_PRINT_ERROR_NOCURRENTPATH);
 
-	return PRINT_CLASS(pc)->fill (pc);
+	gp_gc_close_all (pc->gc);
+
+	if (PRINT_CLASS (pc)->fill)
+		ret = PRINT_CLASS(pc)->fill (pc, ART_WIND_RULE_NONZERO);
+
+	gp_gc_newpath (pc->gc);
+
+	return ret;
 }
 
 int
 gnome_print_eofill (GnomePrintContext *pc)
 {
+	gint ret = GNOME_PRINT_OK;
+
 	g_return_val_if_fail (pc != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (gp_gc_has_currentpath (pc->gc), GNOME_PRINT_ERROR_NOCURRENTPATH);
 
-	return PRINT_CLASS(pc)->eofill (pc);
+	gp_gc_close_all (pc->gc);
+
+	if (PRINT_CLASS (pc)->fill)
+		ret = PRINT_CLASS(pc)->fill (pc, ART_WIND_RULE_ODDEVEN);
+
+	gp_gc_newpath (pc->gc);
+
+	return ret;
 }
 
 int
 gnome_print_setlinewidth (GnomePrintContext *pc, double width)
 {
+	gint ret = GNOME_PRINT_OK;
+
 	g_return_val_if_fail (pc != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (width >= 0, GNOME_PRINT_ERROR_BADVALUE);
 
-	return PRINT_CLASS(pc)->setlinewidth (pc, width);
+	if (PRINT_CLASS (pc)->setlinewidth)
+		ret = PRINT_CLASS(pc)->setlinewidth (pc, width);
+
+	gp_gc_set_linewidth (pc->gc, width);
+
+	return ret;
 }
 
 int
 gnome_print_setmiterlimit (GnomePrintContext *pc, double limit)
 {
+	gint ret = GNOME_PRINT_OK;
+
 	g_return_val_if_fail (pc != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (limit >= 1.0, GNOME_PRINT_ERROR_BADVALUE);
 
-	return PRINT_CLASS(pc)->setmiterlimit (pc, limit);
+	gp_gc_set_miterlimit (pc->gc, limit);
+
+	if (PRINT_CLASS (pc)->setmiterlimit)
+		ret = PRINT_CLASS(pc)->setmiterlimit (pc, limit);
+
+	gp_gc_set_miterlimit (pc->gc, limit);
+
+	return ret;
 }
 
 int
 gnome_print_setlinejoin (GnomePrintContext *pc, int jointype)
 {
+	gint ret = GNOME_PRINT_OK;
+
 	g_return_val_if_fail (pc != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail ((jointype >= 0) && (jointype < 3), GNOME_PRINT_ERROR_BADVALUE);
 
-	return PRINT_CLASS(pc)->setlinejoin (pc, jointype);
+	if (PRINT_CLASS (pc)->setlinejoin)
+		ret = PRINT_CLASS(pc)->setlinejoin (pc, jointype);
+
+	gp_gc_set_linejoin (pc->gc, jointype);
+
+	return ret;
 }
 
 int
 gnome_print_setlinecap (GnomePrintContext *pc, int captype)
 {
+	gint ret = GNOME_PRINT_OK;
+
 	g_return_val_if_fail (pc != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail ((captype >= 0) && (captype < 3), GNOME_PRINT_ERROR_BADVALUE);
 
-	return PRINT_CLASS(pc)->setlinecap (pc, captype);
+	gp_gc_set_linecap (pc->gc, captype);
+
+	if (PRINT_CLASS (pc)->setlinecap)
+		ret = PRINT_CLASS(pc)->setlinecap (pc, captype);
+
+	return ret;
 }
 
 int
 gnome_print_setdash (GnomePrintContext *pc, int n_values, const double *values, double offset)
 {
+	gint ret = GNOME_PRINT_OK;
+
 	g_return_val_if_fail (pc != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
 
-	if (n_values != 0)
-		g_return_val_if_fail (values != NULL, -1);
+	if (n_values != 0) g_return_val_if_fail (values != NULL, GNOME_PRINT_ERROR_BADVALUE);
 
-	return PRINT_CLASS(pc)->setdash (pc, n_values, values, offset);
+	if (PRINT_CLASS (pc)->setdash)
+		ret = PRINT_CLASS(pc)->setdash (pc, n_values, values, offset);
+
+	gp_gc_set_dash (pc->gc, n_values, values, offset);
+
+	return ret;
 }
 
 int
 gnome_print_strokepath (GnomePrintContext *pc)
 {
+	gint ret = GNOME_PRINT_OK;
+
 	g_return_val_if_fail (pc != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (gp_gc_has_currentpath (pc->gc), GNOME_PRINT_ERROR_NOCURRENTPATH);
 
-	return PRINT_CLASS(pc)->strokepath (pc);
+	if (PRINT_CLASS (pc)->strokepath)
+		ret = PRINT_CLASS(pc)->strokepath (pc);
+
+	gp_gc_strokepath (pc->gc);
+
+	return ret;
 }
 
 int
 gnome_print_stroke (GnomePrintContext *pc)
 {
+	gint ret = GNOME_PRINT_OK;
+
 	g_return_val_if_fail (pc != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
 
-	return PRINT_CLASS(pc)->stroke (pc);
+	if (PRINT_CLASS (pc)->stroke)
+		ret = PRINT_CLASS(pc)->stroke (pc);
+
+	gp_gc_newpath (pc->gc);
+
+	return ret;
 }
 
 int
 gnome_print_setfont (GnomePrintContext *pc, GnomeFont *font)
 {
+	gint ret = GNOME_PRINT_OK;
+
 	g_return_val_if_fail (pc != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
-	g_return_val_if_fail (font != NULL, -1);
+	g_return_val_if_fail (font != NULL, GNOME_PRINT_ERROR_BADVALUE);
+	g_return_val_if_fail (GNOME_IS_FONT (font), GNOME_PRINT_ERROR_BADVALUE);
 
-	return PRINT_CLASS(pc)->setfont (pc, font);
+	if (PRINT_CLASS (pc)->setfont)
+		ret = PRINT_CLASS(pc)->setfont (pc, font);
+
+	gp_gc_set_font (pc->gc, font);
+
+	return ret;
 }
 
 int
 gnome_print_show (GnomePrintContext *pc, char const *text)
 {
-	static gboolean uerror = FALSE;
+	g_return_val_if_fail (pc != NULL, -1);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (text != NULL, GNOME_PRINT_ERROR_BADVALUE);
+
+	if (PRINT_CLASS (pc)->show_sized)
+		return (* PRINT_CLASS (pc)->show_sized) (pc, text, strlen (text));
+
+	return GNOME_PRINT_OK;
+}
+
+int
+gnome_print_show_sized (GnomePrintContext *pc, char const *text, int bytes)
+{
+	const char *invalid;
 
 	g_return_val_if_fail (pc != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
 	g_return_val_if_fail (text != NULL, -1);
-	g_return_val_if_fail (!uerror, -1);
 
-	if (strlen (text) < 1) return 0;
+	if (bytes < 1) return GNOME_PRINT_OK;
 
-	if (PRINT_CLASS (pc)->show_ucs4) {
-		static gboolean uinit = FALSE;
-		static unicode_iconv_t uiconv = (unicode_iconv_t) -1;
-		const gchar * ibuf;
-		guint32 * buf;
-		gchar * obuf;
-		size_t ilen, olen;
-		ssize_t converted;
-		gint ret;
+	g_return_val_if_fail (g_utf8_validate (text, bytes, &invalid), GNOME_PRINT_ERROR_TEXTCORRUPT);
 
-		if (!uinit) {
-			unicode_init ();
-			uiconv = unicode_iconv_open ("UCS4", "UTF-8");
-			if (uiconv == (unicode_iconv_t) -1) {
-				g_warning ("Could not initilize unicode convertion");
-				uerror = TRUE;
-				return -1;
-			}
-			uinit = TRUE;
-		}
+	if (PRINT_CLASS (pc)->show_sized)
+		return (* PRINT_CLASS (pc)->show_sized) (pc, text, bytes);
 
-		ret = -1;
-
-		ilen = strlen (text);
-		ibuf = (const gchar *) text;
-		olen = ilen * 2 * sizeof (guint32);
-		buf = g_new (guint32, 2 * ilen);
-		obuf = (gchar *) buf;
-		converted = unicode_iconv (uiconv, &ibuf, &ilen, &obuf, &olen);
-
-		if (converted >= 0) {
-			gint length;
-			length = (guint32 *) obuf - buf;
-			ret = (* PRINT_CLASS (pc)->show_ucs4) (pc, buf, length);
-		}
-		g_free (buf);
-		return ret;
-	}
-
-	return -1;
+	return GNOME_PRINT_OK;
 }
 
+/* WARNING: show_ucs4 is DEPRECATED */
 int
-gnome_print_show_ucs4 (GnomePrintContext * pc, guint32 * buf, gint length)
+gnome_print_show_ucs4 (GnomePrintContext *pc, guint32 *buf, gint length)
 {
+	guchar *utf, *p;
+	gint i;
+
 	g_return_val_if_fail (pc != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
 	g_return_val_if_fail (buf != NULL, -1);
-	g_return_val_if_fail (length > 0, -1);
 
-	if (PRINT_CLASS (pc)->show_ucs4)
-		return (* PRINT_CLASS (pc)->show_ucs4) (pc, buf, length);
+	if (length < 1) return GNOME_PRINT_OK;
 
-	return -1;
+	{
+		static gint warned = FALSE;
+		if (!warned)
+			g_warning ("gnome_print_show_ucs4 is deprecated");
+		warned = TRUE;
+	}
+
+	utf = p = alloca (length * 2);
+
+	for (i = 0; i < length; i++) {
+		p += g_unichar_to_utf8 (GINT_FROM_BE (buf[i]), p);
+	}
+
+	if (PRINT_CLASS (pc)->show_sized)
+		return (* PRINT_CLASS (pc)->show_sized) (pc, utf, p - utf);
+
+	return GNOME_PRINT_OK;
 }
 
 int
 gnome_print_concat (GnomePrintContext *pc, const double matrix[6])
 {
+	gint ret = GNOME_PRINT_OK;
+
 	g_return_val_if_fail (pc != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
 	g_return_val_if_fail (matrix != NULL, -1);
 
-	return PRINT_CLASS(pc)->concat (pc, matrix);
+	if (PRINT_CLASS (pc)->concat)
+		ret = PRINT_CLASS(pc)->concat (pc, matrix);
+
+	gp_gc_concat (pc->gc, matrix);
+
+	return ret;
 }
 
-#if 0
-/**
- * gnome_print_concat_optimize:
- * @gpc: 
- * @: 
- * 
- * This method should be use when apps need to concat a matrix
- * that will be used on all of the page. For example a scaling factor or
- * landscape rotation+translation. No harm is done if this method is a) not
- * used or b) not used correctly. This function provides a mayor optimization
- * when scaling is to be done in the whole page and when the page needs to be
- * landscape rotated.
- **/
-int
-gnome_print_concat_optimize (GnomePrintContext *pc, const double matrix[6])
-{
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
-	g_return_val_if_fail (matrix != NULL, -1);
-
-	g_warning ("DO NOT use this function. It is experimental and will be removed\n");
-	
-	if (PRINT_CLASS (pc)->concat_optimize != NULL)
-		return PRINT_CLASS(pc)->concat_optimize (pc, matrix);
-	else
-		return PRINT_CLASS(pc)->concat (pc, matrix);
-}
-#endif
-
-#ifdef SETMATRIX_DEPRECATED
-int
-gnome_print_setmatrix (GnomePrintContext *pc, const double matrix[6])
-{
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
-	g_return_val_if_fail (matrix != NULL, -1);
-
-	return PRINT_CLASS(pc)->setmatrix (pc, matrix);
-}
-#endif
-
 int
 gnome_print_gsave (GnomePrintContext *pc)
 {
+	gint ret = GNOME_PRINT_OK;
+
 	g_return_val_if_fail (pc != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
 
-	return PRINT_CLASS(pc)->gsave (pc);
+	if (PRINT_CLASS (pc)->gsave)
+		ret = PRINT_CLASS(pc)->gsave (pc);
+
+	gp_gc_gsave (pc->gc);
+
+	return ret;
 }
 
 int
 gnome_print_grestore (GnomePrintContext *pc)
 {
+	gint ret = GNOME_PRINT_OK;
+
 	g_return_val_if_fail (pc != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
 
-	return PRINT_CLASS(pc)->grestore (pc);
+	if (PRINT_CLASS (pc)->grestore)
+		ret = PRINT_CLASS(pc)->grestore (pc);
+
+	gp_gc_grestore (pc->gc);
+
+	return ret;
 }
 
 int
 gnome_print_clip (GnomePrintContext *pc)
 {
+	gint ret = GNOME_PRINT_OK;
+
 	g_return_val_if_fail (pc != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (gp_gc_has_currentpath (pc->gc), GNOME_PRINT_ERROR_NOCURRENTPATH);
 
-	return PRINT_CLASS(pc)->clip (pc);
+	gp_gc_close_all (pc->gc);
+
+	if (PRINT_CLASS (pc)->clip)
+		ret = PRINT_CLASS(pc)->clip (pc, ART_WIND_RULE_NONZERO);
+
+	return ret;
 }
 
 int
 gnome_print_eoclip (GnomePrintContext *pc)
 {
-	return PRINT_CLASS(pc)->eoclip (pc);
+	gint ret = GNOME_PRINT_OK;
+
+	g_return_val_if_fail (pc != NULL, -1);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (gp_gc_has_currentpath (pc->gc), GNOME_PRINT_ERROR_NOCURRENTPATH);
+
+	gp_gc_close_all (pc->gc);
+
+	if (PRINT_CLASS (pc)->clip)
+		ret = PRINT_CLASS(pc)->clip (pc, ART_WIND_RULE_ODDEVEN);
+
+	return ret;
 }
 
 int
@@ -512,7 +615,10 @@ gnome_print_grayimage (GnomePrintContext *pc, const char *data, int width, int h
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
 	g_return_val_if_fail (data != NULL, -1);
 
-	return PRINT_CLASS(pc)->grayimage (pc, data, width, height, rowstride);
+	if (PRINT_CLASS (pc)->grayimage)
+		return PRINT_CLASS(pc)->grayimage (pc, data, width, height, rowstride);
+
+	return GNOME_PRINT_OK;
 }
 
 int
@@ -522,7 +628,10 @@ gnome_print_rgbimage (GnomePrintContext *pc, const char *data, int width, int he
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
 	g_return_val_if_fail (data != NULL, -1);
 
-	return PRINT_CLASS(pc)->rgbimage (pc, data, width, height, rowstride);
+	if (PRINT_CLASS (pc)->rgbimage)
+		return PRINT_CLASS(pc)->rgbimage (pc, data, width, height, rowstride);
+
+	return GNOME_PRINT_OK;
 }
 
 int
@@ -577,18 +686,17 @@ int
 gnome_print_pixbuf (GnomePrintContext *pc, GdkPixbuf *pixbuf)
 {
        if (gdk_pixbuf_get_has_alpha (pixbuf))
-               gnome_print_rgbaimage  (pc,
-                                       gdk_pixbuf_get_pixels    (pixbuf),
-                                       gdk_pixbuf_get_width     (pixbuf),
-                                       gdk_pixbuf_get_height    (pixbuf),
-                                       gdk_pixbuf_get_rowstride (pixbuf));
+               return gnome_print_rgbaimage  (pc,
+					      gdk_pixbuf_get_pixels    (pixbuf),
+					      gdk_pixbuf_get_width     (pixbuf),
+					      gdk_pixbuf_get_height    (pixbuf),
+					      gdk_pixbuf_get_rowstride (pixbuf));
        else
-               gnome_print_rgbimage  (pc,
-                                      gdk_pixbuf_get_pixels    (pixbuf),
-                                      gdk_pixbuf_get_width     (pixbuf),
-                                      gdk_pixbuf_get_height    (pixbuf),
-                                      gdk_pixbuf_get_rowstride (pixbuf));
-	return 1;
+               return gnome_print_rgbimage  (pc,
+					     gdk_pixbuf_get_pixels    (pixbuf),
+					     gdk_pixbuf_get_width     (pixbuf),
+					     gdk_pixbuf_get_height    (pixbuf),
+					     gdk_pixbuf_get_rowstride (pixbuf));
 }
 
 int
@@ -598,16 +706,32 @@ gnome_print_textline (GnomePrintContext *pc, GnomeTextLine *line)
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
 	g_return_val_if_fail (line != NULL, -1);
 
-	return PRINT_CLASS(pc)->textline (pc, line);
+	if (PRINT_CLASS (pc)->textline)
+		return PRINT_CLASS(pc)->textline (pc, line);
+
+	return GNOME_PRINT_OK;
 }
 
 int
 gnome_print_showpage (GnomePrintContext *pc)
 {
+	const GnomeFont * font;
+	gint ret = GNOME_PRINT_OK;
+
 	g_return_val_if_fail (pc != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
 
-	return PRINT_CLASS(pc)->showpage (pc);
+	if (PRINT_CLASS (pc)->showpage)
+		ret = PRINT_CLASS(pc)->showpage (pc);
+
+	/* fixme: do we need to clear the state? */
+	font = gp_gc_get_font (pc->gc);
+	gnome_font_ref (font);
+	gp_gc_reset (pc->gc);
+	gp_gc_set_font (pc->gc, (GnomeFont *) font);
+	gnome_font_unref (font);
+
+	return ret;
 }
 
 int
@@ -616,22 +740,38 @@ gnome_print_beginpage (GnomePrintContext *pc, const char *name_of_this_page)
 	g_return_val_if_fail (pc != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
 
-	return PRINT_CLASS(pc)->beginpage (pc, name_of_this_page);
+	if (PRINT_CLASS (pc)->beginpage)
+		return PRINT_CLASS(pc)->beginpage (pc, name_of_this_page);
+
+	return GNOME_PRINT_OK;
 }
 
 int
 gnome_print_setopacity (GnomePrintContext *pc, double opacity)
 {
+	gint ret = GNOME_PRINT_OK;
+
 	g_return_val_if_fail (pc != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
 
-	return PRINT_CLASS(pc)->setopacity (pc, opacity);
+	if (PRINT_CLASS (pc)->setopacity)
+		ret = PRINT_CLASS(pc)->setopacity (pc, opacity);
+
+	gp_gc_set_opacity (pc->gc, opacity);
+
+	return ret;
 }
 
 int
 gnome_print_context_close (GnomePrintContext *pc)
 {
-	return PRINT_CLASS(pc)->close (pc);
+	g_return_val_if_fail (pc != NULL, -1);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+
+	if (PRINT_CLASS (pc)->close)
+		return PRINT_CLASS(pc)->close (pc);
+
+	return GNOME_PRINT_OK;
 }
 
 
@@ -712,7 +852,7 @@ gnome_print_context_open_file (GnomePrintContext *pc, const char *filename)
 
 /* Return number of bytes written */
 int
-gnome_print_context_write_file (GnomePrintContext *pc, char *buf, size_t size)
+gnome_print_context_write_file (GnomePrintContext *pc, const char *buf, size_t size)
 {
 	g_return_val_if_fail (pc != NULL, 0);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), 0);
@@ -918,7 +1058,8 @@ gnome_print_glyphlist (GnomePrintContext * pc, GnomeGlyphList * glyphlist)
 			for (i = 0; i < string->length; i++) {
 				gdouble r, g, b, a;
 				const gchar *psname;
-				gint ucschar;
+				gchar utf[6];
+				gint len;
 				/* fixme: We need currentpoint here :( */
 				gnome_print_moveto (pc, string->glyphs[i].x, string->glyphs[i].y);
 				r = ((string->glyphs[i].color >> 24) & 0xff) / 255.0;
@@ -928,8 +1069,8 @@ gnome_print_glyphlist (GnomePrintContext * pc, GnomeGlyphList * glyphlist)
 				gnome_print_setrgbcolor (pc, r, g, b);
 				gnome_print_setopacity (pc, a);
 				psname = gnome_font_face_get_glyph_ps_name (face, string->glyphs[i].glyph);
-				ucschar = GINT_TO_BE (gp_unicode_from_ps (psname));
-				gnome_print_show_ucs4 (pc, &ucschar, 1);
+				len = g_unichar_to_utf8 (gp_unicode_from_ps (psname), utf);
+				gnome_print_show_sized (pc, utf, len);
 			}
 		}
 		gnome_pgl_destroy (pgl);
@@ -937,14 +1078,3 @@ gnome_print_glyphlist (GnomePrintContext * pc, GnomeGlyphList * glyphlist)
 
 	return 0;
 }
-
-
-
-
-
-
-
-
-
-
-
