@@ -37,13 +37,16 @@ gnome_printer_load_profiles_from (const char *filename, GnomePrinterProfileList
 	char *base_prefix, *name;
 	int base_len;
 	void *iter;
-	
+
 	base_prefix = alloca (strlen (filename) + 3);
 	base_prefix [0] = '=';
 	strcpy (base_prefix+1, filename);
 	strcat (base_prefix+1, "=");
 	base_len = strlen (base_prefix);
-	
+
+#if 0	/* This requires miguel accepting & applying a gnome-libs patch */
+	gnome_config_ensure_prefix_is_empty ();
+#endif	
 	iter = gnome_config_init_iterator_sections (base_prefix);
 
 	while (gnome_config_iterator_next (iter, &name, NULL)){
@@ -64,11 +67,23 @@ gnome_printer_load_profiles_from (const char *filename, GnomePrinterProfileList
 		pp->mime_type = gnome_config_get_string ("mime-type=");
 		pp->location  = gnome_config_get_string ("location=");
 		pp->output    = gnome_config_get_string ("output=");
+		
 
-		*list = g_list_prepend (*list, pp);
+		/* This is a ugly hack to trap a gnome_config_iterator bug that
+		   I can't replicate anymore. This is what it looks like :
+		   http://www.celorio.com/gnomestuff/gnome-print-bug-profiles.jpg */
+		if (pp != NULL) {
+			if (pp->name[0] == '[')
+				g_warning ("Could not load printer from profile, (iterator bug). Please report it "
+						 "to gnome-print@helixcode.com");
+			else
+				*list = g_list_prepend (*list, pp);
+		}
 
 		gnome_config_pop_prefix ();
 	}
+	
+	*list = g_list_reverse (*list);
 }
 
 static void
@@ -77,7 +92,7 @@ gnome_printer_load_profiles_from_dir (const char *dirname, GnomePrinterProfileLi
 	DIR *dir;
 	struct dirent *dent;
 	const int extlen = sizeof (".profile") - 1;
-	
+
 	dir = opendir (dirname);
 	if (!dir)
 		return;
@@ -97,6 +112,7 @@ gnome_printer_load_profiles_from_dir (const char *dirname, GnomePrinterProfileLi
 		g_free (filename);
 	}
 	closedir (dir);
+
 }
 
 static GnomePrinterProfile *
@@ -127,7 +143,7 @@ gnome_printer_get_profiles (void)
 {
 	GnomePrinterProfileList *list = NULL;
 	char *user_dir;
-	
+
 	gnome_printer_load_profiles_from_dir (PROFILEDIR, &list);
 	user_dir = gnome_util_home_file ("printer-profile");
 	gnome_printer_load_profiles_from_dir (user_dir, &list);
@@ -135,8 +151,9 @@ gnome_printer_get_profiles (void)
 
 	if (list == NULL)
 		list = g_list_prepend (list, gnome_printer_stock_profile ());
-	
+
 	return list;
+	return g_list_reverse (list);
 }
 
 static void
