@@ -1352,12 +1352,17 @@ static GnomeDisplayFont *create_display_font (const char *family,
 					      double points,
 					      double scale)
 {
-  GnomeFont *gnome_font, *closest;
+	GnomeFont *gnome_font;
+#if 0
+  GnomeFont *closest;
+#endif
 #if 0
   GnomeFontUnsized *gfus;
   GnomeDisplayFont *wpdf;
 #else
+#if 0
   GnomeFontFace * face;
+#endif
   GnomeRFont * rfont;
   gdouble transform[6];
 #endif
@@ -1375,7 +1380,10 @@ static GnomeDisplayFont *create_display_font (const char *family,
 
   g_return_val_if_fail (gnome_font != NULL, NULL);
 
+#if 0	/* This is crashing and we don't use */
+          /*face in the rest of the function. (Chema)*/
   face = closest->private->fontmap_entry;
+#endif
 
   art_affine_scale (transform, scale, -scale);
   rfont = gnome_font_get_rfont (gnome_font, transform);
