@@ -5,6 +5,7 @@
 static GHashTable * ps2uni = NULL;
 static GHashTable * uni2ps = NULL;
 static GHashTable * ps2ps = NULL;
+static GHashTable * ps2multi = NULL;
 
 static void gen_tables (void);
 
@@ -36,6 +37,16 @@ gp_const_ps_from_ps (const gchar * name)
 	return g_hash_table_lookup (ps2ps, name);
 }
 
+const GSList *
+gp_multi_from_ps (const gchar * name)
+{
+	g_return_val_if_fail (name != NULL, NULL);
+
+	if (!ps2multi) gen_tables ();
+
+	return g_hash_table_lookup (ps2multi, name);
+}
+
 /*
  * Mapping
  */
@@ -1112,14 +1123,22 @@ gen_tables (void)
 	ps2uni = g_hash_table_new (g_str_hash, g_str_equal);
 	uni2ps = g_hash_table_new (NULL, NULL);
 	ps2ps = g_hash_table_new (g_str_hash, g_str_equal);
+	ps2multi = g_hash_table_new (g_str_hash, g_str_equal);
 
 	g_hash_table_freeze (ps2uni);
 	g_hash_table_freeze (uni2ps);
 	g_hash_table_freeze (ps2ps);
+	g_hash_table_freeze (ps2multi);
 
 	for (i = 0; i < unitabs; i++) {
 		if (!g_hash_table_lookup (ps2uni, unitab[i].name)) {
 			g_hash_table_insert (ps2uni, unitab[i].name, GINT_TO_POINTER (unitab[i].unicode));
+		} else {
+			GSList * l;
+			l = g_hash_table_lookup (ps2multi, unitab[i].name);
+			if (!l) l = g_slist_prepend (l, g_hash_table_lookup (ps2uni, unitab[i].name));
+			l = g_slist_prepend (l, GINT_TO_POINTER (unitab[i].unicode));
+			g_hash_table_insert (ps2multi, unitab[i].name, l);
 		}
 		if (!g_hash_table_lookup (uni2ps, GINT_TO_POINTER (unitab[i].unicode))) {
 			g_hash_table_insert (uni2ps, GINT_TO_POINTER (unitab[i].unicode), unitab[i].name);
@@ -1130,6 +1149,7 @@ gen_tables (void)
 	g_hash_table_thaw (ps2uni);
 	g_hash_table_thaw (uni2ps);
 	g_hash_table_thaw (ps2ps);
+	g_hash_table_thaw (ps2multi);
 }
 
 
