@@ -25,5 +25,9 @@ const gchar * gp_ps_from_unicode (gint unicode);
 
 const gchar * gp_const_ps_from_ps (const gchar * name);
 
+/* Returns GSList of code values, if there are > 1, NULL otherwise */
+
+const GSList * gp_multi_from_ps (const gchar * name);
+
 #endif
 
