@@ -2988,7 +2988,7 @@ glyph_outlines_destroy (Gt1LoadedFont *font)
 }
 
 Gt1LoadedFont *
-gt1_load_font (const char *filename)
+gt1_load_font (const char *filename, GHashTable *privencoding)
 {
   Gt1LoadedFont *loaded_font;
   Gt1Dict *fontdict;
@@ -3007,15 +3007,6 @@ gt1_load_font (const char *filename)
   char *flat;
 
   GHashTable *encoding;
-#if 0
-  Gt1NameId *encode_table_1,
-            *encode_table_2,
-            *encode_table_3,
-            *encode_table_4,
-            *enc_std,
-            *enc_isolatin,
-            *enc_utf8;
-#endif
   int i;
 
   f = fopen (filename, "rb");
@@ -3098,9 +3089,6 @@ gt1_load_font (const char *filename)
     }
   gt1_free (afm_filename);
 
-#if 0
-  enc_isolatin   = gt1_new (Gt1NameId, 1024);
-#endif
   encoding = g_hash_table_new (NULL, NULL);
 
   nc = psc->nc;
@@ -3113,21 +3101,26 @@ gt1_load_font (const char *filename)
 
 	  if (nche->name) {
 		  unival = gp_unicode_from_ps (nche->name);
+		  if (unival < 1) {
+			  if (privencoding) {
+				  unival = GPOINTER_TO_INT (g_hash_table_lookup (privencoding, nche->name));
+			  }
+		  }
 		  if (unival > 0) {
-			  g_hash_table_insert (encoding, GINT_TO_POINTER (unival), GINT_TO_POINTER (nche->Gt1NameId));
+			  if (gp_multi_from_ps (nche->name)) {
+				  const GSList * l;
+				  l = gp_multi_from_ps (nche->name);
+				  while (l) {
+					  g_hash_table_insert (encoding, l->data, GINT_TO_POINTER (nche->Gt1NameId));
+					  l = l->next;
+				  }
+			  } else {
+				  g_hash_table_insert (encoding, GINT_TO_POINTER (unival), GINT_TO_POINTER (nche->Gt1NameId));
+			  }
 		  }
 	  }
   }
 
-#if 0
-  for (i = 0; i < 1024; i++) {
-	  const gchar *name;
-	  name = gp_ps_from_unicode (i);
-	  if (!name) name = ".notdef";
-	  enc_isolatin[i] = gt1_name_context_intern (psc->nc, name);
-  }
-#endif
-
   loaded_font->encoding = encoding;
   loaded_font->id_charstrings = gt1_name_context_intern (psc->nc, "CharStrings");
 
