@@ -1,8 +1,8 @@
 #include <math.h>
 #include <libart_lgpl/art_affine.h>
 #include <gnome.h>
-#include <unicode.h>
 
+#include "libgnomeprint/gp-unicode.h"
 #include "libgnomeprint/gnome-printer.h"
 #include "libgnomeprint/gnome-printer-dialog.h"
 
@@ -31,7 +31,7 @@ static struct poptOption options [] = {
 };
 
 static void do_image (GnomePrintContext * pc, gdouble size, gint alpha);
-static gint latin_to_utf8 (gchar * text, gchar * utext, gint ulength);
+static gint latin_to_utf8 (guchar * text, guchar * utext, gint ulength);
 
 static void
 do_image (GnomePrintContext * pc, gdouble size, gboolean alpha)
@@ -466,30 +466,16 @@ main (int argc, char ** argv)
 }
 
 static gint
-latin_to_utf8 (gchar * text, gchar * utext, gint ulength)
+latin_to_utf8 (guchar * text, guchar * utext, gint ulength)
 {
-	static gboolean uinit = FALSE;
-	static unicode_iconv_t uiconv = (unicode_iconv_t) -1;
-	const gchar * ibuf;
-	gchar * obuf;
-	gint ilen, olen, cnum;
+	guchar * i, * o;
 
-	if (!uinit) {
-		unicode_init ();
-		uiconv = unicode_iconv_open ("UTF-8", "iso-8859-1");
-		g_return_val_if_fail (uiconv != (unicode_iconv_t) -1, -1);
-		uinit = TRUE;
+	o = utext;
+
+	for (i = text; *i; i++) {
+		o += g_unichar_to_utf8 (*i, o);
 	}
 
-	ilen = strlen (text);
-	ibuf = text;
-	olen = ulength;
-	obuf = utext;
-
-	cnum = unicode_iconv (uiconv, &ibuf, &ilen, &obuf, &olen);
-
-	g_return_val_if_fail (cnum >= 0, -1);
-
-	return obuf - utext;
+	return o - utext;
 }
 
